@@ -2,6 +2,7 @@ import collections
 import importlib
 import readline  # noqa: F401
 import shutil
+import sqlite3
 import sys
 import tempfile
 import traceback
@@ -42,10 +43,14 @@ def cli():
 @click.option("--db", "db_path", envvar="ISQLITE_DB")
 @click.argument("table")
 @click.argument("column")
-def main_add_column(db_path, table, column):
+def main_add_column_wrapper(*args, **kwargs):
     """
     Add a column to a table.
     """
+    main_add_column(*args, **kwargs)
+
+
+def main_add_column(db_path, table, column):
     with Database(db_path) as db:
         db.add_column(table, column)
         print(f"Column added to table {table!r}.")
@@ -53,23 +58,64 @@ def main_add_column(db_path, table, column):
 
 @cli.command(name="alter-column")
 @click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
 @click.argument("table")
 @click.argument("column")
-def main_alter_column(db_path, table, column):
+def main_alter_column_wrapper(*args, **kwargs):
     """
     Alter a column's definition.
     """
-    with Database(db_path) as db:
+    return main_alter_column(*args, **kwargs)
+
+
+def main_alter_column(db_path, schema_path, table, column):
+    schema_module = get_schema_module(schema_path)
+    with DatabaseMigrator(db_path, schema_module=schema_module) as migrator:
         column_name, column_def = column.split(maxsplit=1)
-        db.alter_column(table, column_name, column_def)
+        migrator.alter_column(table, column_name, column_def)
         print(f"Column {column_name!r} altered in table {table!r}.")
 
 
 @cli.command(name="create")
 @click.option("--db", "db_path", envvar="ISQLITE_DB")
+@click.argument("table")
+@click.argument("payload", nargs=-1)
+@click.option(
+    "--auto-timestamp",
+    is_flag=True,
+    default=True,
+    help=(
+        "Automatically populate `created_at` and `last_updated_at` columns with "
+        + "current time."
+    ),
+)
+def main_create_wrapper(*args, **kwargs):
+    """
+    Create a new row non-interactively.
+
+    PAYLOAD should be a list of space-separated key-value pairs, e.g.
+
+        isqlite create --db db.sqlite3 my_table a=1 b=2
+    """
+    return main_create(*args, **kwargs)
+
+
+def main_create(db_path, table, payload, *, auto_timestamp=True):
+    payload_as_map = {}
+    for key_value in payload:
+        key, value = key_value.split("=")
+        payload_as_map[key] = value
+
+    with Database(db_path) as db:
+        pk = db.create(table, payload_as_map, auto_timestamp=auto_timestamp)
+        print(f"Row {pk} created.")
+
+
+@cli.command(name="icreate")
+@click.option("--db", "db_path", envvar="ISQLITE_DB")
 @click.option("--schema", "schema_path", envvar="ISQLITE_SCHEMA")
 @click.argument("table")
-def main_create(db_path, schema_path, table):
+def main_icreate(db_path, schema_path, table):
     """
     Create a new row interactively.
     """
@@ -100,12 +146,16 @@ def main_create(db_path, schema_path, table):
 @click.option("--db", "db_path", envvar="ISQLITE_DB")
 @click.argument("table")
 @click.argument("columns", nargs=-1)
-def main_create_table(db_path, table, columns):
+def main_create_table_wrapper(*args, **kwargs):
     """
     Create a table.
     """
+    main_create_table(*args, **kwargs)
+
+
+def main_create_table(db_path, table, columns):
     with Database(db_path) as db:
-        db.create_table(table, *columns)
+        db.create_table(table, columns)
         print(f"Table {table!r} created.")
 
 
@@ -249,24 +299,28 @@ def main_get(db_path, schema_path, table, pk):
 @click.option("--offset", default=None, help=OFFSET_HELP)
 @click.option("--order-by", multiple=True, default=[], help=ORDER_BY_HELP)
 @click.option("--desc", is_flag=True, default=False, help=DESC_HELP)
+def main_list_wrapper(*args, **kwargs):
+    """
+    List the rows in the table, optionally filtered by a SQL clause.
+    """
+    main_list(*args, **kwargs)
+
+
 def main_list(
     db_path,
     schema_path,
     table,
     *,
-    where,
-    search,
-    columns,
-    hide,
-    page,
-    limit,
-    offset,
-    order_by,
-    desc,
+    where=None,
+    search="",
+    columns=[],
+    hide=[],
+    page=1,
+    limit=None,
+    offset=None,
+    order_by=[],
+    desc=False,
 ):
-    """
-    List the rows in the table, optionally filtered by a SQL clause.
-    """
     base_list(
         db_path,
         schema_path,
@@ -301,19 +355,37 @@ def base_list(
     """
     Base implementation shared by `main_list` and `main_search`
     """
-    schema_module = get_schema_module(schema_path)
-    schema_dict = schema_module_to_dict(schema_module)
+    if schema_path:
+        schema_module = get_schema_module(schema_path)
+        schema_dict = schema_module_to_dict(schema_module)
+    else:
+        schema_module = None
+
     with Database(db_path, readonly=True, schema_module=schema_module) as db:
-        rows = db.list(
-            table,
-            where=where,
-            order_by=order_by,
-            limit=limit,
-            offset=offset,
-            descending=desc if order_by else None,
-            # `get_related` is only possible when the database has a schema.
-            get_related=schema_module is not None,
-        )
+        try:
+            rows = db.list(
+                table,
+                where=where,
+                order_by=order_by,
+                limit=limit,
+                offset=offset,
+                descending=desc if order_by else None,
+                # `get_related` is only possible when the database has a schema.
+                get_related=schema_module is not None,
+            )
+        except sqlite3.OperationalError:
+            # Because `get_related` uses SQL joins, it may cause 'ambiguous column'
+            # errors if the user-supplied WHERE clause has unqualified column names. So
+            # we simply retry on error with `get_related=False`.
+            rows = db.list(
+                table,
+                where=where,
+                order_by=order_by,
+                limit=limit,
+                offset=offset,
+                descending=desc if order_by else None,
+                get_related=False,
+            )
 
         for row in rows:
             for key, value in row.items():
@@ -551,6 +623,7 @@ def main_search(
     """
     base_list(
         db_path,
+        schema_path,
         table,
         where=where,
         search=query,
